/* Copyright (c) 1989 by Carrick Sean Casey. All rights reserved. */
/* For copying and distribution information, see the file "copyright.h." */

/* handle various messages from the server */

#include "../copyright.h"
#include <stdio.h>
#include <sys/types.h>
#include <sys/time.h>
#include "../forumnet.h"
#include "externs.h"
#include "../version.h"

/* later, these routines should buffer up text to be sent to the user and */
/* only send them when the user isn't typing */
/* perhaps we should stat the tty and only send when it's been idle for */
/* a second and there is no FIONREAD */
/* DONT stat if there is nothing to send to the tty */

/* open message */

copenmsg(pkt)
char *pkt;
{
	if (split(pkt) != 2) {
		puterr("got bad open message packet");
		return(-1);
	}
	mbreakprint(0, f[0], f[1]);
}

/* personal message */

cpersonalmsg(pkt)
char *pkt;
{
	if (split(pkt) != 2) {
		puterr("got bad personal message packet");
		return(-1);
	}
	mbreakprint(1, f[0], f[1]);
}

/* beep message */

beep(pkt)
char *pkt;
{
	if (split(pkt) != 1) {
		puterr("got bad beep message packet");
		return(-1);
	}
	sprintf(mbuf, "%c[Beep!] %s sent you a beep.", '\007', f[0]);
	putline(mbuf);
}

/* "please drop the connection" message */

exitmsg()
{
	cmd_quit();
}


/* protocol version message */

protomsg(pkt)
char *pkt;
{
	char *p;
	char *getlogin();
	int proto_level;

	if (split(pkt) != 3) {
		puterr("got bad proto message packet");
		return(-1);
	}

	proto_level = atoi(f[0]);
	if (PROTO_LEVEL != proto_level) {
	  fprintf(stderr,
	    "\nSorry, this forum client does not know the same protocol as\n");
	  fprintf(stderr,
	    "the server. I know protocol level %d and the server",PROTO_LEVEL);
	  fprintf(stderr,
	    " is at\nlevel %d.\n\n", proto_level);
	  fprintf(stderr,
	  "You can get the latest client by anonymous ftp from f.ms.uky.edu\n");
	  fprintf(stderr,
	  "[128.163.128.6]. The filename is \"pub/unix/forum.x.x.tar.Z\".\n\n");
	  cmd_quit();
	}

	if (!whoflg) {
		sprintf(mbuf, "Connected to the %s forumnet server (%s).",
		  f[1], f[2]);
		putline(mbuf);
	}

	if (whoflg)
		/* we want a who */
		sendlogin(getlogin(), myname, mygroup, "w", mypass);
	else
		/* we want to log in */
		sendlogin(getlogin(), myname, mygroup, "login", mypass);

	/* clobber the password */
	for (p = mypass; *p != '\0'; p++)
		*p = '\0';
}


/* server is telling us we are logged in */

loginokmsg(pkt)
char *pkt;
{
	connected = 1;
}

/* system status update message */

statusmsg(pkt)
char *pkt;
{
	if (split(pkt) != 2) {
		puterr("got bad status message packet");
		return(-1);
	}
	sprintf(mbuf, "[%s] %s", f[0], f[1]);
	putline(mbuf);
}


/* command output message */
/* this should eventually parse things according to command requested */

cmdoutmsg(pkt)
char *pkt;
{
	char cmd;
	char *idlestr(), *response(), *ampm();
	int fields;

	/* THIS IS REALLY BAD CODING - it should be cleaned up */

	fields = split(pkt);

	if (strcmp(f[0], "wl") == 0) {

		/* body of a who listing */
		sprintf(mbuf, "  %c%-12s  %s  %s  %s  %s@%s  %s",
			*f[1] == '1' ? '+' : ' ', f[2],
			idlestr(f[3]), response(f[4]),
			ampm((time_t)atol(f[5])),
			f[6], f[7], f[8]);
		putline(mbuf);

	} else if (strcmp(f[0], "gh") == 0) {
		if (m_groupheader)
			/* header for a grouplisting */
			putline("Group     ## S  Moderator    Topic");

	} else if (strcmp(f[0], "wh") == 0) {
		if (m_whoheader)
	     /* header for a who listing */
	     putline("   Nickname     Idle  Resp  Sign-On  Account");

	} else if (strcmp(f[0], "wg") == 0) {

		/* group name from a who command */
		if (m_whoheader) {
/*			if (*f[2] == '\0') { */
				sprintf(mbuf, "Group: %s", f[1]);
				putline(mbuf);
/*			} else {
				sprintf(mbuf, "Group: %s  \"%s\"", f[1], f[2]);
				putline(mbuf);
			} */
		} else {
/*			if (*f[2] == '\0') */
				putline(f[1]);
/*			else {
				sprintf(mbuf, "%s  \"%s\"", f[1], f[2]);
				putline(mbuf);
			} */
		}
	} else if (strcmp(f[0], "ch") == 0) {
		listcommands();
	} else if (strcmp(f[0], "c") == 0) {
		sprintf(mbuf, "%c%s", cmdchar, f[1]);
		putline(mbuf);
	} else
		/* just some generic command output */
		putline(f[1]);
}

/* error message */

errormsg(pkt)
char *pkt;
{
	sprintf(mbuf, "[Error] %s", pkt);
	putline(mbuf);
}


/* important system message */

importantmsg(pkt)
char *pkt;
{
	if (split(pkt) != 2) {
		puterr("got bad \"important\" message packet");
		return(-1);
	}
	sprintf(mbuf, "\007[%s] %s", f[0], f[1]);
	putline(mbuf);
}

/* THIS STUFF NEEDS TO BE MOVED! */

/* return a string indicating idle time */

char *
idlestr(num)
char *num;
{
	int seconds = atoi(num);
	static char idletime[8];

	if (seconds >= 60) {
		if (seconds < 3600)
			sprintf(idletime, "%2dm", seconds / 60);
		else
			sprintf(idletime, "%2dh", seconds / 3600);
		return(idletime);
	} else
		return("  -");
}


/* return a string indicating response time */

char *
response(num)
char *num;
{
	int secs = atoi(num);
	static char rtime[8];

	if (secs >= 2) {
		sprintf(rtime, " %2ds", secs);
		return(rtime);
	} else
		return("   -");
}

/* return a time, i.e "7:45pm" */


char *
ampm(secs)
time_t secs;
{
	struct tm *t, *localtime();
	static char timestr[12];
	int hr, mn;

	t = localtime(&secs);
	hr = t->tm_hour;
	mn = t->tm_min;

	if (hr >= 12)
		sprintf(timestr, "%2d:%02dpm", (hr > 12) ? hr - 12 : hr, mn);
	else if (hr > 0)
		sprintf(timestr, "%2d:%02dam", hr, mn);
	else 
		sprintf(timestr, "12:%02dam", mn);

	return(timestr);
}
